@@ -1,6 +1,6 @@
 import calendar
 from collections import defaultdict
-from datetime import datetime
+from datetime import datetime, timedelta
 from time import sleep
 from time import time
 from hashlib import sha256
@@ -20,6 +20,8 @@ from bigquery.errors import (
 BIGQUERY_SCOPE = 'https://www.googleapis.com/auth/bigquery'
 BIGQUERY_SCOPE_READ_ONLY = 'https://www.googleapis.com/auth/bigquery.readonly'
 
+CACHE_TIMEOUT = timedelta(seconds=30)
+
 JOB_CREATE_IF_NEEDED = 'CREATE_IF_NEEDED'
 JOB_CREATE_NEVER = 'CREATE_NEVER'
 JOB_WRITE_TRUNCATE = 'WRITE_TRUNCATE'
@@ -44,7 +46,8 @@ JOB_DESTINATION_FORMAT_CSV = JOB_FORMAT_CSV
 
 
 def get_client(project_id, credentials=None, service_account=None,
-               private_key=None, readonly=True, swallow_results=True):
+               private_key=None, private_key_file=None, readonly=True,
+               swallow_results=True):
     """Return a singleton instance of BigQueryClient. Either
     AssertionCredentials or a service account and private key combination need
     to be provided in order to authenticate requests to BigQuery.
@@ -56,6 +59,9 @@ def get_client(project_id, credentials=None, service_account=None,
         service_account: the Google API service account name.
         private_key: the private key associated with the service account in
                      PKCS12 or PEM format.
+        private_key_file: the name of the file containing the private key
+                          associated with the service account in PKCS12 or PEM
+                          format.
         readonly: bool indicating if BigQuery access is read-only. Has no
                   effect if credentials are provided.
         swallow_results: If set to false then return the actual response value
@@ -65,9 +71,13 @@ def get_client(project_id, credentials=None, service_account=None,
         an instance of BigQueryClient.
     """
 
-    if not credentials and not (service_account and private_key):
-        raise Exception('AssertionCredentials or service account and private'
-                        'key need to be provided')
+    if not credentials:
+        assert service_account and (private_key or private_key_file), \
+            'Must provide AssertionCredentials or service account and key'
+
+    if private_key_file:
+        with open(private_key_file, 'rb') as key_file:
+            private_key = key_file.read()
 
     bq_service = _get_bq_service(credentials=credentials,
                                  service_account=service_account,
@@ -81,7 +91,8 @@ def _get_bq_service(credentials=None, service_account=None, private_key=None,
                     readonly=True):
     """Construct an authorized BigQuery service object."""
 
-    assert credentials or (service_account and private_key)
+    assert credentials or (service_account and private_key), \
+        'Must provide AssertionCredentials or service account and key'
 
     if not credentials:
         scope = BIGQUERY_SCOPE_READ_ONLY if readonly else BIGQUERY_SCOPE
@@ -107,6 +118,7 @@ class BigQueryClient(object):
         self.bigquery = bq_service
         self.project_id = project_id
         self.swallow_results = swallow_results
+        self.cache = {}
 
     def query(self, query, max_results=None, timeout=0, dry_run=False):
         """Submit a query to BigQuery.
@@ -318,7 +330,7 @@ class BigQueryClient(object):
         except HttpError as e:
             logging.error(('Cannot create table {0}.{1}\n'
                            'Http Error: {2}').format(dataset, table,
-                                                     e.message))
+                                                     e.content))
             if self.swallow_results:
                 return False
             else:
@@ -350,7 +362,7 @@ class BigQueryClient(object):
         except HttpError as e:
             logging.error(('Cannot delete table {0}.{1}\n'
                            'Http Error: {2}').format(dataset, table,
-                                                     e.message))
+                                                     e.content))
             if self.swallow_results:
                 return False
             else:
@@ -466,7 +478,7 @@ class BigQueryClient(object):
             configuration['encoding'] = encoding
 
         if schema:
-            configuration['schema'] = schema
+            configuration['schema'] = {'fields': schema}
 
         if source_format:
             configuration['sourceFormat'] = source_format
@@ -790,19 +802,37 @@ class BigQueryClient(object):
                     }]
                 }
 
-    def _get_all_tables(self, dataset_id):
+    def _get_all_tables(self, dataset_id, cache=False):
         """Retrieve a list of all tables for the dataset.
 
         Args:
             dataset_id: the dataset to retrieve table names for.
-
+            cache: To use cached value or not. Timeout value
+                   equals CACHE_TIMEOUT.
         Returns:
             a dictionary of app ids mapped to their table names.
         """
+        do_fetch = True
+        if cache and self.cache.get(dataset_id):
+            time, result = self.cache.get(dataset_id)
+            if datetime.now() - time < CACHE_TIMEOUT:
+                do_fetch = False
 
-        result = self.bigquery.tables().list(
-            projectId=self.project_id,
-            datasetId=dataset_id).execute()
+        if do_fetch:
+            result = self.bigquery.tables().list(
+                projectId=self.project_id,
+                datasetId=dataset_id).execute()
+
+            page_token = result.get('nextPageToken')
+            while page_token:
+                res = self.bigquery.tables().list(
+                    projectId=self.project_id,
+                    datasetId=dataset_id,
+                    pageToken=page_token
+                ).execute()
+                page_token = res.get('nextPageToken')
+                result['tables'] += res.get('tables', [])
+            self.cache[dataset_id] = (datetime.now(), result)
 
         return self._parse_table_list_response(result)
 
