@@ -37,7 +37,7 @@ class TestGetClient(unittest.TestCase):
     def test_no_credentials(self):
         """Ensure an Exception is raised when no credentials are provided."""
 
-        self.assertRaises(Exception, client.get_client, 'foo', 'bar')
+        self.assertRaises(AssertionError, client.get_client, 'foo')
 
     @mock.patch('bigquery.client._credentials')
     @mock.patch('bigquery.client.build')
@@ -99,6 +99,41 @@ class TestGetClient(unittest.TestCase):
         self.assertEquals(mock_bq, bq_client.bigquery)
         self.assertEquals(project_id, bq_client.project_id)
 
+    @mock.patch('bigquery.client._credentials')
+    @mock.patch('bigquery.client.build')
+    @mock.patch('__builtin__.open')
+    def test_initialize_key_file(self, mock_open, mock_build,
+                                 mock_return_cred):
+        """Ensure that a BigQueryClient is initialized and returned with
+        read/write permissions using a private key file.
+        """
+        from bigquery.client import BIGQUERY_SCOPE
+
+        mock_cred = mock.Mock()
+        mock_http = mock.Mock()
+        mock_cred.return_value.authorize.return_value = mock_http
+        mock_bq = mock.Mock()
+        mock_build.return_value = mock_bq
+        key_file = 'key.pem'
+        key = 'key'
+        mock_open.return_value.__enter__.return_value.read.return_value = key
+        service_account = 'account'
+        project_id = 'project'
+        mock_return_cred.return_value = mock_cred
+
+        bq_client = client.get_client(
+            project_id, service_account=service_account,
+            private_key_file=key_file, readonly=False)
+
+        mock_open.assert_called_once_with(key_file, 'rb')
+        mock_return_cred.assert_called_once_with()
+        mock_cred.assert_called_once_with(service_account, key,
+                                          scope=BIGQUERY_SCOPE)
+        mock_cred.authorize.assert_called_once()
+        mock_build.assert_called_once_with('bigquery', 'v2', http=mock_http)
+        self.assertEquals(mock_bq, bq_client.bigquery)
+        self.assertEquals(project_id, bq_client.project_id)
+
 
 class TestQuery(unittest.TestCase):
 
@@ -578,7 +613,7 @@ class TestImportDataFromURIs(unittest.TestCase):
             "configuration": {
                 "load": {
                     "sourceUris": ["sourceuri"],
-                    "schema": ["schema"],
+                    "schema": {"fields": ["schema"]},
                     "destinationTable": {
                         "projectId": self.project_id,
                         "datasetId": self.dataset_id,
@@ -637,7 +672,7 @@ class TestImportDataFromURIs(unittest.TestCase):
             "configuration": {
                 "load": {
                     "sourceUris": ["sourceuri"],
-                    "schema": ["schema"],
+                    "schema": {"fields": ["schema"]},
                     "destinationTable": {
                         "projectId": self.project_id,
                         "datasetId": self.dataset_id,
@@ -732,7 +767,7 @@ class TestImportDataFromURIs(unittest.TestCase):
             "configuration": {
                 "load": {
                     "sourceUris": ["sourceuri"],
-                    "schema": ["schema"],
+                    "schema": {"fields": ["schema"]},
                     "destinationTable": {
                         "projectId": self.project_id,
                         "datasetId": self.dataset_id,
@@ -1028,6 +1063,42 @@ class TestFilterTablesByTime(unittest.TestCase):
         self.assertEqual([], tables)
 
 
+NEXT_TABLE_LIST_RESPONSE = {
+    "kind": "bigquery#tableList",
+    "etag": "\"t_UlB9a9mrx5sjQInRGzeDrLrS0/TsIP_i4gAeLegj84WzkPzBPIkjo\"",
+    "nextPageToken": "2013_05_appspot_1",
+    "tables": [
+        {
+            "kind": "bigquery#table",
+            "id": "project:dataset.2013_06_appspot_10",
+            "tableReference": {
+                "projectId": "project",
+                "datasetId": "dataset",
+                "tableId": "2013_06_appspot_10"
+            }
+        },
+        {
+            "kind": "bigquery#table",
+            "id": "project:dataset.2013_06_appspot_11",
+            "tableReference": {
+                "projectId": "project",
+                "datasetId": "dataset",
+                "tableId": "2013_06_appspot_11"
+            }
+        },
+        {
+            "kind": "bigquery#table",
+            "id": "project:dataset.2013_06_appspot_12",
+            "tableReference": {
+                "projectId": "project",
+                "datasetId": "dataset",
+                "tableId": "2013_06_appspot_12"
+            }
+        },
+    ],
+    "totalItems": 3
+}
+
 FULL_TABLE_LIST_RESPONSE = {
     "kind": "bigquery#tableList",
     "etag": "\"GSclnjk0zID1ucM3F-xYinOm1oE/cn58Rpu8v8pB4eoJQaiTe11lPQc\"",
@@ -1663,7 +1734,83 @@ class TestGetAllTables(unittest.TestCase):
             'appspot': {'2013_05_appspot': 1367366400}
         }
 
-        tables = bq._get_all_tables('dataset')
+        tables = bq._get_all_tables('dataset', cache=False)
+        self.assertEquals(expected_result, tables)
+
+    def test_get_all_tables_with_page_token(self):
+        """Ensure get_all_tables fetches all tables names from BigQuery"""
+
+        mock_execute = mock.Mock()
+        mock_execute.execute.side_effect = [NEXT_TABLE_LIST_RESPONSE,
+                                            FULL_TABLE_LIST_RESPONSE]
+
+        mock_tables = mock.Mock()
+        mock_tables.list.return_value = mock_execute
+
+        mock_bq_service = mock.Mock()
+        mock_bq_service.tables.return_value = mock_tables
+
+        bq = client.BigQueryClient(mock_bq_service, 'project')
+
+        expected_result = {
+            'appspot-3': {'2013_06_appspot_3': 1370044800},
+            'appspot-2': {'2013_06_appspot_2': 1370044800},
+            'appspot-1': {'2013_06_appspot_1': 1370044800},
+            'appspot-6': {'appspot_6_2013_06': 1370044800},
+            'appspot-5': {'2013_06_appspot_5': 1370044800},
+            'appspot-4': {'2013_06_appspot_4': 1370044800},
+            'appspot': {'2013_05_appspot': 1367366400},
+            'appspot-10': {'2013_06_appspot_10': 1370044800},
+            'appspot-12': {'2013_06_appspot_12': 1370044800},
+            'appspot-11': {'2013_06_appspot_11': 1370044800},
+        }
+        tables = bq._get_all_tables('dataset', cache=False)
+        self.assertEquals(expected_result, tables)
+
+    def test_get_all_tables_with_cache(self):
+        """Ensure get_all_tables uses cache when fetching"""
+        mock_execute = mock.Mock()
+        mock_execute.execute.return_value = FULL_TABLE_LIST_RESPONSE
+
+        mock_tables = mock.Mock()
+        mock_tables.list.return_value = mock_execute
+
+        mock_bq_service = mock.Mock()
+        mock_bq_service.tables.return_value = mock_tables
+
+        bq = client.BigQueryClient(mock_bq_service, 'project')
+
+        expected_result = {
+            'appspot-3': {'2013_06_appspot_3': 1370044800},
+            'appspot-2': {'2013_06_appspot_2': 1370044800},
+            'appspot-1': {'2013_06_appspot_1': 1370044800},
+            'appspot-6': {'appspot_6_2013_06': 1370044800},
+            'appspot-5': {'2013_06_appspot_5': 1370044800},
+            'appspot-4': {'2013_06_appspot_4': 1370044800},
+            'appspot': {'2013_05_appspot': 1367366400}
+        }
+
+        tables = bq._get_all_tables('dataset', cache=True)
+        self.assertEquals(expected_result, tables)
+
+        mock_execute.execute.side_effect = [NEXT_TABLE_LIST_RESPONSE,
+                                            FULL_TABLE_LIST_RESPONSE]
+        tables = bq._get_all_tables('dataset', cache=True)
+        self.assertEquals(expected_result, tables)
+
+        expected_result = {
+            'appspot-3': {'2013_06_appspot_3': 1370044800},
+            'appspot-2': {'2013_06_appspot_2': 1370044800},
+            'appspot-1': {'2013_06_appspot_1': 1370044800},
+            'appspot-6': {'appspot_6_2013_06': 1370044800},
+            'appspot-5': {'2013_06_appspot_5': 1370044800},
+            'appspot-4': {'2013_06_appspot_4': 1370044800},
+            'appspot': {'2013_05_appspot': 1367366400},
+            'appspot-10': {'2013_06_appspot_10': 1370044800},
+            'appspot-12': {'2013_06_appspot_12': 1370044800},
+            'appspot-11': {'2013_06_appspot_11': 1370044800},
+        }
+        tables = bq._get_all_tables('dataset', cache=False)
         self.assertEquals(expected_result, tables)
 
 
